@@ -73,14 +73,18 @@ struct ReadGroupInfoIterData {
     matvar_t *matvar;
 };
 
-#if defined(H5Rdereference)
-/* HDF5 1.10.0 */
+#if H5_VERSION_GE(1,10,0)
 #define H5RDEREFERENCE(obj_id, ref_type, _ref) H5Rdereference2((obj_id), H5P_DATASET_ACCESS_DEFAULT, (ref_type), (_ref))
 #else
-/* HDF5 prior to 1.10.0 */
 #define H5RDEREFERENCE(obj_id, ref_type, _ref) H5Rdereference((obj_id), (ref_type), (_ref))
 #endif
 
+#if H5_VERSION_GE(1,10,3)
+#define H5OGET_INFO_BY_NAME(loc_id, name, oinfo, lapl_id) H5Oget_info_by_name2((loc_id), (name), (oinfo), H5O_INFO_BASIC, (lapl_id));
+#else
+#define H5OGET_INFO_BY_NAME(loc_id, name, oinfo, lapl_id) H5Oget_info_by_name((loc_id), (name), (oinfo), (lapl_id));
+#endif
+
 #if !defined(MAX_RANK)
 /* Maximal number of dimensions for stack allocated temporary dimension arrays */
 #define MAX_RANK (3)
@@ -550,7 +554,7 @@ Mat_H5ReadDims(hid_t dset_id, hsize_t *numel, int *rank)
             int k;
             (void)H5Sget_simple_extent_dims(space_id,dims,NULL);
             *numel = 1;
-            /* Permutate dimensions */
+            /* Permute dimensions */
             for ( k = 0; k < *rank; k++ ) {
                 perm_dims[k] = (size_t)dims[*rank - k - 1];
                 *numel *= perm_dims[k];
@@ -562,7 +566,7 @@ Mat_H5ReadDims(hid_t dset_id, hsize_t *numel, int *rank)
                 int k;
                 (void)H5Sget_simple_extent_dims(space_id,dims,NULL);
                 *numel = 1;
-                /* Permutate dimensions */
+                /* Permute dimensions */
                 for ( k = 0; k < *rank; k++ ) {
                     perm_dims[k] = (size_t)dims[*rank - k - 1];
                     *numel *= perm_dims[k];
@@ -656,17 +660,19 @@ Mat_H5ReadDatasetInfo(mat_t *mat,matvar_t *matvar,hid_t dset_id)
     /* If the dataset is a cell array read the info of the cells */
     if ( MAT_C_CELL == matvar->class_type ) {
         matvar_t **cells;
-        int i,ncells = 1;
+        size_t ncells = 1;
+        int j;
         hobj_ref_t *ref_ids;
 
-        for ( i = 0; i < matvar->rank; i++ )
-            ncells *= matvar->dims[i];
+        for ( j = 0; j < matvar->rank; j++ )
+            ncells *= matvar->dims[j];
         matvar->data_size = sizeof(matvar_t**);
         matvar->nbytes    = ncells*matvar->data_size;
         matvar->data      = malloc(matvar->nbytes);
         cells = (matvar_t**)matvar->data;
 
         if ( ncells ) {
+            size_t i;
             ref_ids = (hobj_ref_t*)malloc(ncells*sizeof(*ref_ids));
             H5Dread(dset_id,H5T_STD_REF_OBJ,H5S_ALL,H5S_ALL,H5P_DEFAULT,ref_ids);
             for ( i = 0; i < ncells; i++ ) {
@@ -782,7 +788,7 @@ Mat_H5ReadGroupInfo(mat_t *mat,matvar_t *matvar,hid_t dset_id)
 
     if ( nfields > 0 ) {
         H5O_info_t object_info;
-        H5Oget_info_by_name(dset_id, matvar->internal->fieldnames[0], &object_info, H5P_DEFAULT);
+        H5OGET_INFO_BY_NAME(dset_id, matvar->internal->fieldnames[0], &object_info, H5P_DEFAULT);
         obj_type = object_info.type;
     } else {
         obj_type = H5O_TYPE_UNKNOWN;
@@ -862,7 +868,7 @@ Mat_H5ReadGroupInfo(mat_t *mat,matvar_t *matvar,hid_t dset_id)
         for ( k = 0; k < nfields; k++ ) {
             H5O_info_t object_info;
             fields[k] = NULL;
-            H5Oget_info_by_name(dset_id, matvar->internal->fieldnames[k], &object_info, H5P_DEFAULT);
+            H5OGET_INFO_BY_NAME(dset_id, matvar->internal->fieldnames[k], &object_info, H5P_DEFAULT);
             if ( object_info.type == H5O_TYPE_DATASET ) {
                 field_id = H5Dopen(dset_id,matvar->internal->fieldnames[k],
                                    H5P_DEFAULT);
@@ -922,7 +928,7 @@ Mat_H5ReadGroupInfoIterate(hid_t dset_id, const char *name, const H5L_info_t *in
 
     /* FIXME: follow symlinks, datatypes? */
 
-    H5Oget_info_by_name(dset_id, name, &object_info, H5P_DEFAULT);
+    H5OGET_INFO_BY_NAME(dset_id, name, &object_info, H5P_DEFAULT);
     if ( H5O_TYPE_DATASET != object_info.type && H5O_TYPE_GROUP != object_info.type )
         return 0;
 
@@ -1073,7 +1079,7 @@ Mat_H5ReadNextReferenceData(hid_t ref_id,matvar_t *matvar,mat_t *mat)
                 Mat_VarRead73(mat,matvar);
             } else {
                 matvar_t **fields;
-                int i,nfields = 0;
+                size_t i,nfields = 0;
 
                 if ( !matvar->nbytes || !matvar->data_size || NULL == matvar->data )
                     break;
@@ -1290,7 +1296,7 @@ Mat_VarWriteEmpty(hid_t id,matvar_t *matvar,const char *name,const char* class_n
  * @param matvar pointer to the cell array variable
  * @param name Name of the HDF dataset
  * @param refs_id pointer to the id of the /#refs# group in HDF5
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @retval 0 on success
  * @endif
  */
@@ -1362,7 +1368,7 @@ Mat_VarWriteCell73(hid_t id,matvar_t *matvar,const char *name,hid_t *refs_id,hsi
  * @param id HDF id of the parent object
  * @param matvar pointer to the character variable
  * @param name Name of the HDF dataset
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @retval 0 on success
  * @endif
  */
@@ -1490,7 +1496,7 @@ Mat_WriteEmptyVariable73(hid_t id,const char *name,hsize_t rank,size_t *dims)
  * @param id HDF id of the parent object
  * @param matvar pointer to the logical variable
  * @param name Name of the HDF dataset
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @retval 0 on success
  * @endif
  */
@@ -1505,7 +1511,7 @@ Mat_VarWriteLogical73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims)
         numel *= dims[k];
     }
 
-    if ( matvar->compression ) {
+    if ( matvar->compression == MAT_COMPRESSION_ZLIB ) {
         plist = H5Pcreate(H5P_DATASET_CREATE);
         if ( MAX_RANK >= matvar->rank ) {
             hsize_t chunk_dims[MAX_RANK];
@@ -1578,7 +1584,7 @@ Mat_VarWriteLogical73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims)
  * @param id HDF id of the parent object
  * @param matvar pointer to the numeric variable
  * @param name Name of the HDF dataset
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @param max_dims maximum dimensions
  * @retval 0 on success
  * @endif
@@ -1611,7 +1617,7 @@ Mat_VarWriteNumeric73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims,h
                 return -1;
             }
         }
-        if ( matvar->compression )
+        if ( matvar->compression == MAT_COMPRESSION_ZLIB )
             H5Pset_deflate(plist, 9);
     } else {
         plist = H5P_DEFAULT;
@@ -1658,7 +1664,7 @@ Mat_VarWriteNumeric73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims,h
  * @param id HDF id of the parent object
  * @param matvar pointer to the numeric variable
  * @param name Name of the HDF dataset
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @param dim dimension to append data
  * @retval 0 on success
  * @endif
@@ -1666,8 +1672,7 @@ Mat_VarWriteNumeric73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims,h
 static int
 Mat_VarWriteAppendNumeric73(hid_t id,matvar_t *matvar,const char *name,hsize_t *dims,int dim)
 {
-    int err = 0;
-    unsigned long k;
+    int err = 0, k;
     hsize_t numel = 1;
 
     for ( k = 0; k < matvar->rank; k++ ) {
@@ -1821,7 +1826,7 @@ Mat_VarWriteSparse73(hid_t id,matvar_t *matvar,const char *name)
  * @param matvar pointer to the structure variable
  * @param name Name of the HDF dataset
  * @param refs_id pointer to the id of the /#refs# group in HDF5
- * @param dims array of permutated dimensions
+ * @param dims array of permuted dimensions
  * @param max_dims maximum dimensions
  * @retval 0 on success
  * @endif
@@ -1968,8 +1973,7 @@ Mat_VarWriteStruct73(hid_t id,matvar_t *matvar,const char *name,hid_t *refs_id,h
 static int
 Mat_VarWriteAppendStruct73(hid_t id,matvar_t *matvar,const char *name,hid_t *refs_id,hsize_t *dims,int dim)
 {
-    int err = 0;
-    unsigned long k;
+    int err = 0, k;
     hsize_t nmemb = 1;
 
     for ( k = 0; k < matvar->rank; k++ ) {
@@ -2080,7 +2084,7 @@ Mat_VarWriteNextType73(hid_t id,matvar_t *matvar,const char *name,hid_t *refs_id
 {
     int err = -1, k;
 
-    /* Permutate dimensions */
+    /* Permute dimensions */
     for ( k = 0; k < matvar->rank; k++ ) {
         dims[k] = matvar->dims[matvar->rank - k - 1];
     }
@@ -2131,7 +2135,7 @@ Mat_VarWriteAppendNextType73(hid_t id,matvar_t *matvar,const char *name,hid_t *r
 {
     int err = -1, k;
 
-    /* Permutate dimensions */
+    /* Permute dimensions */
     for ( k = 0; k < matvar->rank; k++ ) {
         dims[k] = matvar->dims[matvar->rank - k - 1];
     }
@@ -2192,23 +2196,30 @@ Mat_Create73(const char *matname,const char *hdr_str)
     mat_t *mat = NULL;
     size_t err;
     time_t t;
-    hid_t plist_id,fid;
+    hid_t plist_id,fid,plist_ap;
 
     plist_id = H5Pcreate(H5P_FILE_CREATE);
     H5Pset_userblock(plist_id,512);
-    fid = H5Fcreate(matname,H5F_ACC_TRUNC,plist_id,H5P_DEFAULT);
+    plist_ap = H5Pcreate(H5P_FILE_ACCESS);
+#if H5_VERSION_GE(1,10,2)
+    H5Pset_libver_bounds(plist_ap,H5F_LIBVER_EARLIEST,H5F_LIBVER_V18);
+#endif
+    fid = H5Fcreate(matname,H5F_ACC_TRUNC,plist_id,plist_ap);
     H5Fclose(fid);
     H5Pclose(plist_id);
 
     fp = fopen(matname,"r+b");
-    if ( !fp )
+    if ( !fp ) {
+        H5Pclose(plist_ap);
         return NULL;
+    }
 
     (void)fseek(fp,0,SEEK_SET);
 
     mat = (mat_t*)malloc(sizeof(*mat));
     if ( mat == NULL ) {
         fclose(fp);
+        H5Pclose(plist_ap);
         return NULL;
     }
 
@@ -2255,7 +2266,8 @@ Mat_Create73(const char *matname,const char *hdr_str)
 
     fclose(fp);
 
-    fid = H5Fopen(matname,H5F_ACC_RDWR,H5P_DEFAULT);
+    fid = H5Fopen(matname,H5F_ACC_RDWR,plist_ap);
+    H5Pclose(plist_ap);
 
     mat->fp = malloc(sizeof(hid_t));
     *(hid_t*)mat->fp = fid;
@@ -2355,7 +2367,7 @@ Mat_VarRead73(mat_t *mat,matvar_t *matvar)
         case MAT_C_STRUCT:
         {
             matvar_t **fields;
-            int i,nfields = 0;
+            size_t i,nfields = 0;
 
             if ( !matvar->internal->num_fields || NULL == matvar->data )
                 break;
@@ -2378,7 +2390,7 @@ Mat_VarRead73(mat_t *mat,matvar_t *matvar)
         case MAT_C_CELL:
         {
             matvar_t **cells;
-            int i,ncells = 0;
+            size_t i,ncells = 0;
 
             ncells = matvar->nbytes / matvar->data_size;
             cells  = (matvar_t**)matvar->data;
@@ -2704,7 +2716,7 @@ Mat_VarReadNextInfoIterate(hid_t id, const char *name, const H5L_info_t *info, v
     if ( 0 == strcmp(name, "#refs#") || 0 == strcmp(name, "#subsystem#") )
         return 0;
 
-    H5Oget_info_by_name(id, name, &object_info, H5P_DEFAULT);
+    H5OGET_INFO_BY_NAME(id, name, &object_info, H5P_DEFAULT);
     if ( H5O_TYPE_DATASET != object_info.type && H5O_TYPE_GROUP != object_info.type )
         return 0;
 
