@@ -34,13 +34,13 @@
 #define MATIO_MINOR_VERSION 5
 
 /* Matio release level number */
-#define MATIO_RELEASE_LEVEL 12
+#define MATIO_RELEASE_LEVEL 13
 
 /* Matio version number */
-#define MATIO_VERSION 1512
+#define MATIO_VERSION 1513
 
 /* Matio version string */
-#define MATIO_VERSION_STR "1.5.12"
+#define MATIO_VERSION_STR "1.5.13"
 
 /* Default file format */
 #define MAT_FT_DEFAULT MAT_FT_MAT5
@@ -166,4 +166,13 @@
 #define MATIO_NORETURN
 #endif
 
+/*
+  The following macros handle format attributes for type-checks against a
+  format string.
+*/
+
+#define MATIO_FORMATATTR_PRINTF1
+#define MATIO_FORMATATTR_PRINTF2
+#define MATIO_FORMATATTR_VPRINTF
+
 #endif /* MATIO_PUBCONF_H */
